@@ -134,6 +134,7 @@ class MultiHeadAttention(Layer):
         self,
         heads,
         head_size,
+        out_dim=None,
         key_size=None,
         use_bias=True,
         attention_scale=True,
@@ -143,7 +144,7 @@ class MultiHeadAttention(Layer):
         super(MultiHeadAttention, self).__init__(**kwargs)
         self.heads = heads
         self.head_size = head_size
-        self.out_dim = heads * head_size
+        self.out_dim = out_dim or heads * head_size
         self.key_size = key_size or head_size
         self.use_bias = use_bias
         self.attention_scale = attention_scale
@@ -162,7 +163,7 @@ class MultiHeadAttention(Layer):
             kernel_initializer=self.kernel_initializer
         )
         self.v_dense = Dense(
-            units=self.out_dim,
+            units=self.head_size * self.heads,
             use_bias=self.use_bias,
             kernel_initializer=self.kernel_initializer
         )
@@ -222,7 +223,7 @@ class MultiHeadAttention(Layer):
         o = tf.einsum('bhjk,bkhd->bjhd', a, vw)
         if p_bias == 'typical_relative':
             o = o + tf.einsum('bhjk,jkd->bjhd', a, pos_embeddings)
-        o = K.reshape(o, (-1, K.shape(o)[1], self.out_dim))
+        o = K.reshape(o, (-1, K.shape(o)[1], self.head_size * self.heads))
         o = self.o_dense(o)
         # 返回结果
         o = sequence_masking(o, q_mask, 0)
@@ -231,13 +232,15 @@ class MultiHeadAttention(Layer):
     def compute_output_shape(self, input_shape):
         return (input_shape[0][0], input_shape[0][1], self.out_dim)
 
-    def compute_mask(self, inputs, mask):
-        return mask[0]
+    def compute_mask(self, inputs, mask=None):
+        if mask is not None:
+            return mask[0]
 
     def get_config(self):
         config = {
             'heads': self.heads,
             'head_size': self.head_size,
+            'out_dim': self.out_dim,
             'key_size': self.key_size,
             'use_bias': self.use_bias,
             'attention_scale': self.attention_scale,
@@ -558,7 +561,10 @@ class RelativePositionEmbeddingT5(RelativePositionEmbedding):
 
 
 class FeedForward(Layer):
-    """FeedForward层，其实就是两个Dense层的叠加
+    """FeedForward层
+    如果activation不是一个list，那么它就是两个Dense层的叠加；如果activation是
+    一个list，那么第一个Dense层将会被替换成门控线性单元（Gated Linear Unit）。
+    参考论文: https://arxiv.org/abs/2002.05202
     """
     def __init__(
         self,
@@ -570,7 +576,9 @@ class FeedForward(Layer):
     ):
         super(FeedForward, self).__init__(**kwargs)
         self.units = units
-        self.activation = activations.get(activation)
+        if not isinstance(activation, list):
+            activation = [activation]
+        self.activation = [activations.get(act) for act in activation]
         self.use_bias = use_bias
         self.kernel_initializer = initializers.get(kernel_initializer)
 
@@ -579,13 +587,16 @@ class FeedForward(Layer):
         super(FeedForward, self).build(input_shape)
         output_dim = input_shape[-1]
 
-        self.dense_1 = Dense(
-            units=self.units,
-            activation=self.activation,
-            use_bias=self.use_bias,
-            kernel_initializer=self.kernel_initializer
-        )
-        self.dense_2 = Dense(
+        for i, activation in enumerate(self.activation):
+            i_dense = Dense(
+                units=self.units,
+                activation=activation,
+                use_bias=self.use_bias,
+                kernel_initializer=self.kernel_initializer
+            )
+            setattr(self, 'i%s_dense' % i, i_dense)
+
+        self.o_dense = Dense(
             units=output_dim,
             use_bias=self.use_bias,
             kernel_initializer=self.kernel_initializer
@@ -593,15 +604,18 @@ class FeedForward(Layer):
 
     @recompute_grad
     def call(self, inputs):
-        x = inputs
-        x = self.dense_1(x)
-        x = self.dense_2(x)
+        x = self.i0_dense(inputs)
+        for i in range(1, len(self.activation)):
+            x = x * getattr(self, 'i%s_dense' % i)(inputs)
+        x = self.o_dense(x)
         return x
 
     def get_config(self):
         config = {
             'units': self.units,
-            'activation': activations.serialize(self.activation),
+            'activation': [
+                activations.serialize(act) for act in self.activation
+            ],
             'use_bias': self.use_bias,
             'kernel_initializer':
                 initializers.serialize(self.kernel_initializer),
@@ -630,9 +644,13 @@ class ConditionalRandomField(Layer):
         )
         if self.lr_multiplier != 1:
             K.set_value(self._trans, K.eval(self._trans) / self.lr_multiplier)
-            self.trans = self.lr_multiplier * self._trans
+
+    @property
+    def trans(self):
+        if self.lr_multiplier != 1:
+            return self.lr_multiplier * self._trans
         else:
-            self.trans = self._trans
+            return self._trans
 
     def compute_mask(self, inputs, mask=None):
         return None
@@ -756,9 +774,6 @@ class MaximumEntropyMarkovModel(Layer):
                     self._trans,
                     K.eval(self._trans) / self.lr_multiplier
                 )
-                self.trans = self.lr_multiplier * self._trans
-            else:
-                self.trans = self._trans
         else:
             self._l_trans = self.add_weight(
                 name='l_trans',
@@ -778,14 +793,31 @@ class MaximumEntropyMarkovModel(Layer):
                     self._l_trans,
                     K.eval(self._l_trans) / self.lr_multiplier
                 )
-                self.l_trans = self.lr_multiplier * self._l_trans
                 K.set_value(
                     self._r_trans,
                     K.eval(self._r_trans) / self.lr_multiplier
                 )
-                self.r_trans = self.lr_multiplier * self._r_trans
-            else:
-                self.l_trans, self.r_trans = self._l_trans, self._r_trans
+
+    @property
+    def trans(self):
+        if self.lr_multiplier != 1:
+            return self.lr_multiplier * self._trans
+        else:
+            return self._trans
+
+    @property
+    def l_trans(self):
+        if self.lr_multiplier != 1:
+            return self.lr_multiplier * self._l_trans
+        else:
+            return self._l_trans
+
+    @property
+    def r_trans(self):
+        if self.lr_multiplier != 1:
+            return self.lr_multiplier * self._r_trans
+        else:
+            return self._r_trans
 
     def compute_mask(self, inputs, mask=None):
         return None
@@ -934,6 +966,14 @@ class Loss(Layer):
         else:
             return input_shape[self.output_axis]
 
+    def compute_mask(self, inputs, mask):
+        if self.output_axis is None:
+            return mask
+        elif isinstance(self.output_axis, list):
+            return [mask[i] for i in self.output_axis]
+        else:
+            return mask[self.output_axis]
+
     def get_config(self):
         config = {
             'output_axis': self.output_axis,
