@@ -7,6 +7,7 @@ import numpy as np
 import re
 import sys
 from collections import defaultdict
+import json
 
 _open_ = open
 is_py2 = six.PY2
@@ -14,6 +15,16 @@ is_py2 = six.PY2
 if not is_py2:
     basestring = str
 
+    
+def to_array(*args):
+    """批量转numpy的array
+    """
+    results = [np.array(a) for a in args]
+    if len(args) == 1:
+        return results[0]
+    else:
+        return results
+
 
 def is_string(s):
     """判断是否是字符串
@@ -119,7 +130,7 @@ def parallel_apply(
     输出可能是func(c), func(a), func(b)。
     参数：
         dummy: False是多进程/线性，True则是多线程/线性；
-        callback: 处理单个输出的回调函数；
+        callback: 处理单个输出的回调函数。
     """
     if dummy:
         from multiprocessing.dummy import Pool, Queue
@@ -131,9 +142,9 @@ def parallel_apply(
     def worker_step(in_queue, out_queue):
         # 单步函数包装成循环执行
         while True:
-            d = in_queue.get()
+            i, d = in_queue.get()
             r = func(d)
-            out_queue.put(r)
+            out_queue.put((i, r))
 
     # 启动多进程/线程
     pool = Pool(workers, worker_step, (in_queue, out_queue))
@@ -145,21 +156,21 @@ def parallel_apply(
     def process_out_queue():
         out_count = 0
         for _ in range(out_queue.qsize()):
-            d = out_queue.get()
+            i, d = out_queue.get()
             out_count += 1
             if callback is None:
-                results.append(d)
+                results.append((i, d))
             else:
                 callback(d)
         return out_count
 
     # 存入数据，取出结果
     in_count, out_count = 0, 0
-    for d in iterable:
+    for i, d in enumerate(iterable):
         in_count += 1
         while True:
             try:
-                in_queue.put(d, block=False)
+                in_queue.put((i, d), block=False)
                 break
             except six.moves.queue.Full:
                 out_count += process_out_queue()
@@ -172,7 +183,8 @@ def parallel_apply(
     pool.terminate()
 
     if callback is None:
-        return results
+        results = sorted(results, key=lambda r: r[0])
+        return [r[1] for r in results]
 
 
 def sequence_padding(inputs, length=None, padding=0):
@@ -282,9 +294,9 @@ class DataGenerator(object):
     def __iter__(self, random=False):
         raise NotImplementedError
 
-    def forfit(self):
+    def forfit(self, random=True):
         while True:
-            for d in self.__iter__(True):
+            for d in self.__iter__(random):
                 yield d
 
 
@@ -349,37 +361,43 @@ class AutoRegressiveDecoder(object):
             self.first_output_ids = np.array([[self.start_id]])
 
     @staticmethod
-    def set_rtype(default='probas'):
-        """用来给predict方法加上rtype参数，并作相应的处理
+    def wraps(default_rtype='probas', use_states=False):
+        """用来进一步完善predict函数
+        目前包含：1. 设置rtype参数，并做相应处理；
+                  2. 确定states的使用，并做相应处理。
         """
         def actual_decorator(predict):
-            def new_predict(self, inputs, output_ids, step, rtype=default):
+            def new_predict(
+                self, inputs, output_ids, states, rtype=default_rtype
+            ):
                 assert rtype in ['probas', 'logits']
-                result = predict(self, inputs, output_ids, step)
-                if default == 'probas':
-                    if rtype == 'probas':
-                        return result
-                    else:
-                        return np.log(result + 1e-12)
+                prediction = predict(self, inputs, output_ids, states)
+
+                if not use_states:
+                    prediction = (prediction, None)
+
+                if default_rtype == 'logits':
+                    prediction = (softmax(prediction[0]), prediction[1])
+
+                if rtype == 'probas':
+                    return prediction
                 else:
-                    if rtype == 'probas':
-                        return softmax(result, -1)
-                    else:
-                        return result
+                    return np.log(prediction[0] + 1e-12), prediction[1]
 
             return new_predict
 
         return actual_decorator
 
-    def predict(self, inputs, output_ids, step, rtype='logits'):
+    def predict(self, inputs, output_ids, states=None, rtype='logits'):
         """用户需自定义递归预测函数
-        rtype为字符串logits或probas，用户定义的时候，应当根据rtype来
-        返回不同的结果，rtype=probas时返回归一化的概率，rtype=logits时
-        则返回softmax前的结果或者概率对数。
+        说明：rtype为字符串logits或probas，用户定义的时候，应当根据rtype来
+              返回不同的结果，rtype=probas时返回归一化的概率，rtype=logits时
+              则返回softmax前的结果或者概率对数。
+        返回：二元组 (得分或概率, states)
         """
         raise NotImplementedError
 
-    def beam_search(self, inputs, topk):
+    def beam_search(self, inputs, topk, states=None, min_ends=1):
         """beam search解码
         说明：这里的topk即beam size；
         返回：最优解码序列。
@@ -387,7 +405,9 @@ class AutoRegressiveDecoder(object):
         inputs = [np.array([i]) for i in inputs]
         output_ids, output_scores = self.first_output_ids, np.zeros(1)
         for step in range(self.maxlen):
-            scores = self.predict(inputs, output_ids, step, 'logits')  # 计算当前得分
+            scores, states = self.predict(
+                inputs, output_ids, states, 'logits'
+            )  # 计算当前得分
             if step == 0:  # 第1步预测后将输入重复topk次
                 inputs = [np.repeat(i, topk, axis=0) for i in inputs]
             scores = output_scores.reshape((-1, 1)) + scores  # 综合累积得分
@@ -399,21 +419,25 @@ class AutoRegressiveDecoder(object):
             output_scores = np.take_along_axis(
                 scores, indices, axis=None
             )  # 更新得分
+            end_counts = (output_ids == self.end_id).sum(1)  # 统计出现的end标记
             if output_ids.shape[1] >= self.minlen:  # 最短长度判断
                 best_one = output_scores.argmax()  # 得分最大的那个
-                if indices_2[best_one, 0] == self.end_id:  # 如果已经终止
+                if end_counts[best_one] == min_ends:  # 如果已经终止
                     return output_ids[best_one]  # 直接输出
                 else:  # 否则，只保留未完成部分
-                    flag = (indices_2[:, 0] != self.end_id)  # 标记未完成序列
+                    flag = (end_counts < min_ends)  # 标记未完成序列
                     if not flag.all():  # 如果有已完成的
                         inputs = [i[flag] for i in inputs]  # 扔掉已完成序列
                         output_ids = output_ids[flag]  # 扔掉已完成序列
                         output_scores = output_scores[flag]  # 扔掉已完成序列
+                        end_counts = end_counts[flag]  # 扔掉已完成end计数
                         topk = flag.sum()  # topk相应变化
         # 达到长度直接输出
         return output_ids[output_scores.argmax()]
 
-    def random_sample(self, inputs, n, topk=None, topp=None):
+    def random_sample(
+        self, inputs, n, topk=None, topp=None, states=None, min_ends=1
+    ):
         """随机采样n个结果
         说明：非None的topk表示每一步只从概率最高的topk个中采样；而非None的topp
              表示每一步只从概率最高的且概率之和刚好达到topp的若干个token中采样。
@@ -423,7 +447,9 @@ class AutoRegressiveDecoder(object):
         output_ids = self.first_output_ids
         results = []
         for step in range(self.maxlen):
-            probas = self.predict(inputs, output_ids, step, 'probas')  # 计算当前概率
+            probas, states = self.predict(
+                inputs, output_ids, states, 'probas'
+            )  # 计算当前概率
             probas /= probas.sum(axis=1, keepdims=True)  # 确保归一化
             if step == 0:  # 第1步预测后将结果重复n次
                 probas = np.repeat(probas, n, axis=0)
@@ -454,14 +480,16 @@ class AutoRegressiveDecoder(object):
                     k_indices, sample_ids, axis=1
                 )  # 对齐原id
             output_ids = np.concatenate([output_ids, sample_ids], 1)  # 更新输出
+            end_counts = (output_ids == self.end_id).sum(1)  # 统计出现的end标记
             if output_ids.shape[1] >= self.minlen:  # 最短长度判断
-                flag = (sample_ids[:, 0] == self.end_id)  # 标记已完成序列
+                flag = (end_counts == min_ends)  # 标记已完成序列
                 if flag.any():  # 如果有已完成的
                     for ids in output_ids[flag]:  # 存好已完成序列
                         results.append(ids)
                     flag = (flag == False)  # 标记未完成序列
                     inputs = [i[flag] for i in inputs]  # 只保留未完成部分输入
                     output_ids = output_ids[flag]  # 只保留未完成部分候选集
+                    end_counts = end_counts[flag]  # 只保留未完成部分end计数
                     if len(output_ids) == 0:
                         break
         # 如果还有未完成序列，直接放入结果
@@ -550,6 +578,89 @@ def longest_common_subsequence(source, target):
     return l, mapping[::-1]
 
 
+class WebServing(object):
+    """简单的Web接口
+    用法：
+        arguments = {'text': (None, True), 'n': (int, False)}
+        web = WebServing(port=8864)
+        web.route('/gen_synonyms', gen_synonyms, arguments)
+        web.start()
+        # 然后访问 http://127.0.0.1:8864/gen_synonyms?text=你好
+    说明：
+        基于bottlepy简单封装，仅作为临时测试使用，不保证性能。
+        目前仅保证支持 Tensorflow 1.x + Keras <= 2.3.1。
+        欢迎有经验的开发者帮忙改进。
+    依赖：
+        pip install bottle
+        pip install paste
+        （如果不用 server='paste' 的话，可以不装paste库）
+    """
+    def __init__(self, host='0.0.0.0', port=8000, server='paste'):
+
+        import tensorflow as tf
+        from bert4keras.backend import K
+        import bottle
+
+        self.host = host
+        self.port = port
+        self.server = server
+        self.graph = tf.get_default_graph()
+        self.sess = K.get_session()
+        self.set_session = K.set_session
+        self.bottle = bottle
+
+    def wraps(self, func, arguments, method='GET'):
+        """封装为接口函数
+        参数：
+            func：要转换为接口的函数，需要保证输出可以json化，即需要
+                  保证 json.dumps(func(inputs)) 能被执行成功；
+            arguments：声明func所需参数，其中key为参数名，value[0]为
+                       对应的转换函数（接口获取到的参数值都是字符串
+                       型），value[1]为该参数是否必须；
+            method：GET或者POST。
+        """
+        def new_func():
+            outputs = {'code': 0, 'desc': u'succeeded', 'data': {}}
+            kwargs = {}
+            for key, value in arguments.items():
+                if method == 'GET':
+                    result = self.bottle.request.GET.get(key)
+                else:
+                    result = self.bottle.request.POST.get(key)
+                if result is None:
+                    if value[1]:
+                        outputs['code'] = 1
+                        outputs['desc'] = 'lack of "%s" argument' % key
+                        return json.dumps(outputs, ensure_ascii=False)
+                else:
+                    if value[0] is None:
+                        result = convert_to_unicode(result)
+                    else:
+                        result = value[0](result)
+                    kwargs[key] = result
+            try:
+                with self.graph.as_default():
+                    self.set_session(self.sess)
+                    outputs['data'] = func(**kwargs)
+            except Exception as e:
+                outputs['code'] = 2
+                outputs['desc'] = str(e)
+            return json.dumps(outputs, ensure_ascii=False)
+
+        return new_func
+
+    def route(self, path, func, arguments, method='GET'):
+        """添加接口
+        """
+        func = self.wraps(func, arguments, method)
+        self.bottle.route(path, method=method)(func)
+
+    def start(self):
+        """启动服务
+        """
+        self.bottle.run(host=self.host, port=self.port, server=self.server)
+
+
 class Hook:
     """注入uniout模块，实现import时才触发
     """
